use std::{fs::File, io::BufReader};

#[derive(serde::Deserialize, Debug)]
pub(crate) struct Config {
    pub(crate) matrix: MatrixConfig,
    pub(crate) meshtastic: MeshtasticConfig,
    #[serde(default = "get_db_uri")]
    pub(crate) db: String,
}

#[derive(serde::Deserialize, Debug)]
pub(crate) struct MatrixConfig {
    pub(crate) username: String,
    #[serde(default = "get_matrix_password")]
    pub(crate) password: String,
    #[serde(default = "get_matrix_device_name")]
    pub(crate) device_name: String,
    pub(crate) room: String,
    #[serde(default = "get_matrix_state_dir")]
    pub(crate) state_dir: String,
    #[serde(default = "get_matrix_state_passphrase")]
    pub(crate) state_passphrase: String,
}

#[derive(serde::Deserialize, Debug)]
pub(crate) struct MeshtasticConfig {
    pub(crate) device: String,
}

fn get_matrix_device_name() -> String {
    "meshtastic-bridge".to_string()
}

fn get_db_uri() -> String {
    "sqlite://state/matrix-meshtastic-bridge.db".to_string()
}

fn get_matrix_password() -> String {
    match std::env::var("MATRIX_PASSWORD") {
        Ok(p) => p,
        Err(_) => "".to_string(),
    }
}

fn get_matrix_state_dir() -> String {
    "state".to_string()
}

fn get_matrix_state_passphrase() -> String {
    match std::env::var("MATRIX_STATE_PASSPHRASE") {
        Ok(p) => p,
        Err(_) => "a".to_string(), // I'm sorry but encryption at rest is dumb if the decryption key is stored in the same place as the encrypted data
    }
}

pub(crate) async fn read_config() -> Config {
    let filename = match std::env::var("CONFIG_PATH") {
        Ok(value) => value,
        Err(_) => "config.json".to_string(),
    };

    log::info!("loading config file from {}", filename);

    let file = File::open(filename).expect("failed to read config file");
    let reader = BufReader::new(file);

    // Read the JSON contents of the file as an instance of `User`.
    serde_json::from_reader(reader).expect("failed to parse config.json")
}
